import * as R from "ramda";
import React from "react";
import { mount } from "enzyme";
import { createContext } from "./providers";
import { connect } from "./consumers";
import counterModule from "Examples/ui-kit/Counter/index";
import { actions as CounterActions, getters as CounterGetters } from "Examples/ui-kit/Counter/model";
describe("## Redux-MVC connect", function () {
  it("Should pass the context instance as props", function () {
    var Div = connect()(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div"
        }, props.instanceId)
      );
    });
    var App = createContext({
      module: counterModule
    })(function (_ref) {
      var children = _ref.children;
      return (
        /*#__PURE__*/
        React.createElement("div", null, children)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, {
      instanceId: "contextInstanceId"
    },
    /*#__PURE__*/
    React.createElement(Div, null)));
    expect(wrapper.find("#div").text()).toBe("contextInstanceId");
  });
  it("Should pass the default instance as props", function () {
    var Div = connect()(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div"
        }, props.instanceId)
      );
    });
    var App = createContext({
      module: counterModule
    })(function (_ref2) {
      var children = _ref2.children;
      return (
        /*#__PURE__*/
        React.createElement("div", null, children)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, null,
    /*#__PURE__*/
    React.createElement(Div, null)));
    expect(wrapper.find("#div").text()).toBe("default");
  });
  it("Should pass the connected state", function () {
    var Div = connect({
      count: CounterGetters.count
    })(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var App = createContext({
      module: counterModule
    })(function (_ref3) {
      var children = _ref3.children;
      return (
        /*#__PURE__*/
        React.createElement("div", null, children)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, null,
    /*#__PURE__*/
    React.createElement(Div, null)));
    var instance = wrapper.instance();
    instance.store.dispatch(CounterActions.add());
    expect(wrapper.find("#div").text()).toBe("1");
  });
  it("Should re-render when the state is updated", function () {
    var spy = jest.fn(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var Div = connect({
      count: CounterGetters.count
    })(spy);
    var App = createContext({
      module: counterModule
    })(function (_ref4) {
      var children = _ref4.children;
      return (
        /*#__PURE__*/
        React.createElement("div", null, children)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, null,
    /*#__PURE__*/
    React.createElement(Div, null)));
    expect(wrapper.find("#div").text()).toBe("0");
    expect(spy.mock.calls.length).toBe(1);
    var instance = wrapper.instance();
    instance.store.dispatch(CounterActions.add());
    expect(wrapper.find("#div").text()).toBe("1");
    expect(spy.mock.calls.length).toBe(2);
  });
  it("Should re-render if props change", function () {
    var spy = jest.fn(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var decorate = R.compose(createContext({
      module: counterModule
    }), connect());
    var Div = decorate(spy);
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(Div, {
      count: 0
    }));
    expect(spy.mock.calls.length).toBe(1);
    expect(wrapper.find("#div").text()).toBe("0");
    wrapper.setProps({
      count: 1
    });
    expect(wrapper.find("#div").text()).toBe("1");
    expect(spy.mock.calls.length).toBe(2);
  });
  it("Should not re-render when the state is updated but the stateProps don't change", function () {
    var spy = jest.fn(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var Div = connect({
      count: CounterGetters.count
    })(spy);
    var App = createContext({
      module: counterModule
    })(function (_ref5) {
      var children = _ref5.children;
      return (
        /*#__PURE__*/
        React.createElement("div", null, children)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, null,
    /*#__PURE__*/
    React.createElement(Div, {
      instanceId: "counter"
    })));
    expect(wrapper.find("#div").text()).toBe("0");
    expect(spy.mock.calls.length).toBe(1);
    var instance = wrapper.instance();
    instance.store.dispatch(CounterActions.add(null, {
      meta: {
        instanceId: "counter2"
      }
    }));
    expect(wrapper.find("#div").text()).toBe("0");
    expect(spy.mock.calls.length).toBe(1);
  });
  it("Should pass the connected state from the context instance", function () {
    var Div = connect({
      count: CounterGetters.count
    })(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var App = createContext({
      module: counterModule
    })(function (_ref6) {
      var children = _ref6.children;
      return (
        /*#__PURE__*/
        React.createElement("div", null, children)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, {
      instanceId: "instance1"
    },
    /*#__PURE__*/
    React.createElement(Div, null)));
    var instance = wrapper.instance();
    instance.store.dispatch(CounterActions.add(null, {
      meta: {
        instanceId: "instance1"
      }
    }));
    expect(wrapper.find("#div").text()).toBe("1");
  });
  it("Should pass the connected state from the instance prop", function () {
    var Div = connect({
      count: CounterGetters.count
    })(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var App = createContext({
      module: counterModule
    })(function (_ref7) {
      var children = _ref7.children;
      return (
        /*#__PURE__*/
        React.createElement("div", null, children)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, {
      instanceId: "instance1"
    },
    /*#__PURE__*/
    React.createElement(Div, {
      instanceId: "instance2"
    })));
    var instance = wrapper.instance();
    instance.store.dispatch(CounterActions.add(null, {
      meta: {
        instanceId: "instance2"
      }
    }));
    expect(wrapper.find("#div").text()).toBe("1");
  });
  it("Should pass the connected action", function () {
    var Div = connect(null, {
      onClick: CounterActions.add
    })(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div",
          onClick: props.onClick
        })
      );
    });
    var App = createContext({
      module: counterModule
    })(function (_ref8) {
      var children = _ref8.children;
      return (
        /*#__PURE__*/
        React.createElement("div", null, children)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, null,
    /*#__PURE__*/
    React.createElement(Div, null)));
    wrapper.find("#div").simulate("click");
    var instance = wrapper.instance();
    expect(instance.store.getState().Counter).toEqual({
      "default": {
        count: 1
      }
    });
  });
  it("Should dispatch to the context instance", function () {
    var Div = connect(null, {
      onClick: CounterActions.add
    })(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div",
          onClick: props.onClick
        })
      );
    });
    var App = createContext({
      module: counterModule
    })(function (_ref9) {
      var children = _ref9.children;
      return (
        /*#__PURE__*/
        React.createElement("div", null, children)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, {
      instanceId: "instance1"
    },
    /*#__PURE__*/
    React.createElement(Div, null)));
    wrapper.find("#div").simulate("click");
    var instance = wrapper.instance();
    expect(instance.store.getState().Counter).toEqual({
      "default": {
        count: 0
      },
      instance1: {
        count: 1
      }
    });
  });
  it("Should dispatch to the instance prop", function () {
    var Div = connect(null, {
      onClick: CounterActions.add
    })(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div",
          onClick: props.onClick
        })
      );
    });
    var App = createContext({
      module: counterModule
    })(function (_ref10) {
      var children = _ref10.children;
      return (
        /*#__PURE__*/
        React.createElement("div", null, children)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, {
      instanceId: "instance1"
    },
    /*#__PURE__*/
    React.createElement(Div, {
      instanceId: "instance2"
    })));
    wrapper.find("#div").simulate("click");
    var instance = wrapper.instance();
    expect(instance.store.getState().Counter).toEqual({
      "default": {
        count: 0
      },
      instance2: {
        count: 1
      }
    });
  });
  it("should call getters if props change", function () {
    var fixedCount = jest.fn(function () {
      return 10;
    });
    var Div = connect({
      fixedCount: fixedCount
    })(function (props) {
      return (
        /*#__PURE__*/
        React.createElement("div", {
          id: "div"
        }, props.text)
      );
    });
    var App = createContext({
      module: counterModule
    })(function (_ref11) {
      var text = _ref11.text;
      return (
        /*#__PURE__*/
        React.createElement(Div, {
          text: text
        })
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(App, {
      text: "hola"
    }));
    expect(wrapper.find("#div").text()).toBe("hola");
    expect(fixedCount.mock.calls.length).toBe(1);
    wrapper.setProps({
      text: "chau"
    });
    expect(wrapper.find("#div").text()).toBe("chau");
    expect(fixedCount.mock.calls.length).toBe(2);
  });
});