import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import { createStore, applyMiddleware, compose } from "redux";
import { mergeAll, prop, path, pathOr, getActionInstanceId, uniq, applyBridgeMiddleware, pick, ensureAncestorRender } from "./utils";
import { GLOBAL_CONTEXT_ID, GLOBAL_UPDATE, DEFAULT_INSTANCE_ID } from "./constants";
import { makeBridgeMiddleware } from "./middleware";
export var addReducer = function addReducer() {
  return function (module) {
    var dependencies = Object.values(module.dependencies || {});
    var iniState = mergeAll([module.iniState].concat(_toConsumableArray(dependencies.map(function (dep) {
      return dep.iniState || {};
    }))));
    var reducers = mergeAll([module.reducers].concat(_toConsumableArray(dependencies.map(function (dep) {
      return dep.reducers || {};
    }))));
    return _objectSpread({}, module, {
      iniState: iniState,
      reducers: reducers,
      reducer: function reducer(state, action) {
        if (state === undefined) {
          return iniState;
        }

        var globalUpdate = path(["meta", GLOBAL_UPDATE], action);

        if (globalUpdate) {
          return mergeAll([state, globalUpdate]);
        }

        var selectedReducer = reducers[action.type];

        if (typeof selectedReducer !== "function") {
          return state;
        }

        var instanceId = getActionInstanceId(action, pathOr( // if the action is not in dependencies then must be an action from the root module
        module.singleton, [action.namespace, "singleton"], module.dependencies));
        var namespace = action.namespace;
        var p = [namespace, instanceId];
        var oldState = pathOr(path([namespace, DEFAULT_INSTANCE_ID], iniState), p, state);
        var newState = selectedReducer(oldState, action);

        if (newState !== oldState) {
          return mergeAll([state, _defineProperty({}, namespace, mergeAll([state[namespace], _defineProperty({}, instanceId, mergeAll([oldState, newState]))]))]);
        }

        return state;
      }
    });
  };
};

var defaultCompose = function defaultCompose() {
  return compose;
};

var composeEnhancers = process.env.NODE_ENV !== "production" && window && // eslint-disable-next-line no-underscore-dangle
window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ || defaultCompose;
var created = Symbol("created");
export var addLifecycle = function addLifecycle() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return function (module) {
    return _objectSpread({}, module, {
      constructor: function constructor(_ref3) {
        var _ref3$persist = _ref3.persist,
            persist = _ref3$persist === void 0 ? true : _ref3$persist,
            moduleInstances = _ref3.moduleInstances,
            contextId = _ref3.contextId;
        var moduleInstance = moduleInstances[contextId];

        if (!moduleInstance[created] || !persist) {
          moduleInstance[created] = true;
          var middleware = Object.values(moduleInstance.middleware || {});
          var globalInstance = contextId === GLOBAL_CONTEXT_ID ? moduleInstance : moduleInstances[GLOBAL_CONTEXT_ID];

          if (applyBridgeMiddleware({
            moduleInstance: moduleInstance,
            globalInstance: globalInstance
          })) {
            //create bridge middleware
            moduleInstance.bridgeMiddleware = makeBridgeMiddleware({
              moduleInstance: moduleInstance,
              globalInstance: globalInstance
            });
            middleware.push(moduleInstance.bridgeMiddleware);
          }

          var iniState = moduleInstance.iniState;

          if (contextId !== GLOBAL_CONTEXT_ID && prop("store", globalInstance)) {
            iniState = mergeAll([moduleInstance.iniState, pick(moduleInstance.trackGlobalNamespaces || [], globalInstance.store.getState())]);
          } //create store


          moduleInstance.store = ensureAncestorRender(createStore(moduleInstance.reducer, iniState, composeEnhancers(_objectSpread({
            name: moduleInstance.namespace,
            serialize: {
              replacer: function replacer(key, value) {
                if (value && value.dispatchConfig && value._targetInst // eslint-disable-line
                ) {
                    return "[EVENT]";
                  }

                return value;
              }
            }
          }, options)).apply(void 0, [applyMiddleware.apply(void 0, _toConsumableArray(middleware))].concat(_toConsumableArray(moduleInstance.enhancers || [])))));
        }

        moduleInstance.store.subscribeExecute();

        if (moduleInstance.bridgeMiddleware) {
          moduleInstance.bridgeMiddleware.bind();
        }

        return moduleInstance;
      },
      componentWillUnmount: function componentWillUnmount(_ref4) {
        var moduleInstances = _ref4.moduleInstances,
            contextId = _ref4.contextId;
        var moduleInstance = moduleInstances[contextId]; //stop middleware

        if (moduleInstance.bridgeMiddleware) {
          moduleInstance.bridgeMiddleware.unbind();
        }

        moduleInstance.store.unsubscribeExecute();
        return moduleInstance;
      }
    });
  };
};
export var merge = function merge(left) {
  return function (right) {
    var dependencies = _objectSpread({}, left.dependencies, {}, right.dependencies, _defineProperty({}, left.namespace, left));

    return _objectSpread({}, right, {
      dependencies: dependencies
    });
  };
};

var makeDispatchToGlobal = function makeDispatchToGlobal(namespaces) {
  var re = new RegExp("(".concat(namespaces.join("|"), ")/"));
  return function (action) {
    return !re.test(action.type);
  };
};

export var addBridge = function addBridge() {
  var _ref5 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      trackGlobalNamespaces = _ref5.trackGlobalNamespaces,
      dispatchToGlobal = _ref5.dispatchToGlobal;

  return function (module) {
    return _objectSpread({}, module, {
      trackGlobalNamespaces: uniq(Object.values(module.dependencies || {}).reduce(function (trackGlobalNamespaces, module) {
        return [].concat(_toConsumableArray(trackGlobalNamespaces), _toConsumableArray(module.trackGlobalNamespaces || []));
      }, Array.isArray(trackGlobalNamespaces) ? trackGlobalNamespaces : [])),
      dispatchToGlobal: typeof dispatchToGlobal === "function" ? dispatchToGlobal : makeDispatchToGlobal(uniq([].concat(_toConsumableArray(Object.values(module.dependencies || {}).map(function (val) {
        return val.namespace;
      })), [module.namespace])))
    });
  };
};
export var createModule = compose(addLifecycle(), addReducer());