import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import _toPropertyKey from "@babel/runtime/helpers/esm/toPropertyKey";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { pick, diff, has, noop } from "./utils";
import { GLOBAL_UPDATE } from "./constants";

var globalUpdate = function globalUpdate(action, state) {
  return _objectSpread({}, action, {
    meta: _objectSpread({}, action.meta || {}, _defineProperty({}, GLOBAL_UPDATE, state))
  });
};

export var makeBridgeMiddleware = function makeBridgeMiddleware(_ref) {
  var moduleInstance = _ref.moduleInstance,
      globalInstance = _ref.globalInstance;
  var middleware = noop;

  if (moduleInstance !== globalInstance) {
    // eslint-disable-next-line no-unused-vars
    middleware = function middleware(store) {
      return function (next) {
        return function (action) {
          if (moduleInstance.enableDispatchToGlobal && !has(["meta", GLOBAL_UPDATE], action) && moduleInstance.dispatchToGlobal(action)) {
            return globalInstance && globalInstance.store && globalInstance.store.dispatch(action);
          }

          return next(action);
        };
      };
    };
  } else {
    middleware = function middleware(store) {
      var lastState = store.getState();
      return function (next) {
        return function (action) {
          var nextAction = next(action);
          var listeners = Object.values(globalInstance.listeners || {});

          if (listeners.length === 0) {
            return nextAction;
          }

          var nextState = store.getState();

          if (diff(lastState, nextState)) {
            listeners.forEach(function (_ref2) {
              var dispatch = _ref2.dispatch,
                  trackGlobalNamespaces = _ref2.trackGlobalNamespaces;

              if (diff(lastState, nextState, trackGlobalNamespaces)) {
                dispatch(globalUpdate(action, pick(trackGlobalNamespaces, nextState)));
              }
            });
            lastState = nextState;
          }

          return nextAction;
        };
      };
    };
  }

  middleware.bind = function () {
    if (moduleInstance !== globalInstance) {
      moduleInstance.enableDispatchToGlobal = typeof moduleInstance.dispatchToGlobal === "function";

      if (moduleInstance.trackGlobalNamespaces && moduleInstance.trackGlobalNamespaces.length) {
        globalInstance.listeners = _objectSpread({}, globalInstance.listeners || {}, _defineProperty({}, moduleInstance.namespace, {
          trackGlobalNamespaces: moduleInstance.trackGlobalNamespaces,
          dispatch: moduleInstance.store.dispatch
        }));
      }
    }
  };

  middleware.unbind = function () {
    if (moduleInstance !== globalInstance) {
      moduleInstance.enableDispatchToGlobal = false; // eslint-disable-next-line no-unused-vars

      var _ref3 = globalInstance.listeners || {},
          _moduleInstance$names = moduleInstance.namespace,
          remove = _ref3[_moduleInstance$names],
          listeners = _objectWithoutProperties(_ref3, [_moduleInstance$names].map(_toPropertyKey));

      globalInstance.listeners = listeners;
    }
  };

  return middleware;
};