import React from "react";
import { mount } from "enzyme";
import { createContext } from "./providers";
import { StoreManager } from "./context";
import { DEFAULT_INSTANCE_ID } from "./constants";
import counterModule from "Examples/ui-kit/Counter/index";
describe("## Redux-MVC context", function () {
  it("Should pass the default instance id", function () {
    var Div = createContext({
      module: counterModule
    })(function (_ref) {
      var instanceId = _ref.instanceId;
      return (
        /*#__PURE__*/
        React.createElement("div", null, instanceId)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(Div, null));
    expect(wrapper.text()).toContain(DEFAULT_INSTANCE_ID);
  });
  it("Should pass the instance id prop", function () {
    var Div = createContext({
      module: counterModule
    })(function (_ref2) {
      var instanceId = _ref2.instanceId;
      return (
        /*#__PURE__*/
        React.createElement("div", null, instanceId)
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(Div, {
      instanceId: "instanceId"
    }));
    expect(wrapper.text()).toContain("instanceId");
  });
  it("Should set the current contextId", function () {
    var Div = createContext({
      module: counterModule,
      contextId: "context"
    })(function () {
      return (
        /*#__PURE__*/
        React.createElement(StoreManager.Consumer, null, function (value) {
          return value.contextId;
        })
      );
    });
    var wrapper = mount(
    /*#__PURE__*/
    React.createElement(Div, null));
    expect(wrapper.text()).toContain("context");
  });
  it("Should create the module instance", function () {
    var moduleInstances;
    var Div = createContext({
      module: counterModule,
      contextId: "context"
    })(function () {
      return (
        /*#__PURE__*/
        React.createElement(StoreManager.Consumer, null, function (value) {
          moduleInstances = value.moduleInstances;
        })
      );
    });
    mount(
    /*#__PURE__*/
    React.createElement(Div, null));
    expect(moduleInstances).toEqual(expect.objectContaining({
      context: expect.any(Object)
    }));
  });
  it("Should create the store", function () {
    var moduleInstances;
    var Div = createContext({
      module: counterModule,
      contextId: "context"
    })(function () {
      return (
        /*#__PURE__*/
        React.createElement(StoreManager.Consumer, null, function (value) {
          moduleInstances = value.moduleInstances;
        })
      );
    });
    mount(
    /*#__PURE__*/
    React.createElement(Div, null));
    expect(moduleInstances.context.store).toEqual(expect.objectContaining({
      getState: expect.any(Function),
      subscribe: expect.any(Function)
    }));
  });
});