import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import _toPropertyKey from "@babel/runtime/helpers/esm/toPropertyKey";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _typeof from "@babel/runtime/helpers/esm/typeof";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";

/* eslint-disable no-underscore-dangle */
import { DEFAULT_INSTANCE_ID } from "./constants"; // lib

export var identity = function identity(x) {
  return x;
};
export var noop = function noop() {};
export var propOr = function propOr(or, prop, val) {
  if (val && typeof val[prop] !== "undefined" && val[prop] !== null) {
    return val[prop];
  }

  return or;
};
export var prop = function prop(_prop, val) {
  if (val) {
    return val[_prop];
  }

  return undefined;
};
export var pathOr = function pathOr(or, path, val) {
  var result = path.reduce(function (acc, key) {
    return prop(key, acc);
  }, val);
  return typeof result !== "undefined" && result !== null ? result : or;
};
export var path = function path(_path, val) {
  return _path.reduce(function (acc, key) {
    return prop(key, acc);
  }, val);
};
var EMPTY = Symbol("empty");
export var has = function has() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return pathOr.apply(void 0, [EMPTY].concat(args)) !== EMPTY;
};
export var mergeAll = function mergeAll(all) {
  return all.reduce(function (acc, val) {
    return _objectSpread({}, acc, {}, val || {});
  }, {});
}; // mvc utils

export var getSelectorInstanceId = function getSelectorInstanceId(props, singleton) {
  return singleton ? DEFAULT_INSTANCE_ID : propOr(DEFAULT_INSTANCE_ID, "instanceId", props);
};
export var getActionInstanceId = function getActionInstanceId(action, singleton) {
  return singleton ? DEFAULT_INSTANCE_ID : pathOr(DEFAULT_INSTANCE_ID, ["meta", "instanceId"], action);
};
export var getDisplayName = function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || "Component";
};
export var capitalizeFirst = function capitalizeFirst(str) {
  return str[0].toUpperCase() + str.slice(1);
};
export var isEmpty = function isEmpty(obj) {
  return !obj || Object.keys(obj).length === 0;
};
export var diff = function diff(oldObj, newObj, keys) {
  if (oldObj === newObj) {
    return false;
  }

  if (_typeof(oldObj) === "object" && _typeof(newObj) === "object") {
    if (Array.isArray(keys)) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = keys[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var key = _step.value;

          if (oldObj[key] !== newObj[key]) {
            return true;
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator["return"] != null) {
            _iterator["return"]();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return false;
    }

    if (Object.keys(oldObj).length !== Object.keys(newObj).length) {
      return true;
    }

    for (var _key2 in oldObj) {
      if (oldObj[_key2] !== newObj[_key2]) {
        return true;
      }
    }
  }

  return false;
};
export var uniq = function uniq(arr) {
  return Object.values(arr.reduce(function (acc, key) {
    return _objectSpread({}, acc, _defineProperty({}, key, key));
  }, {}));
};
export var pick = function pick(arr, obj) {
  return Array.isArray(arr) && obj ? arr.reduce(function (pick, key) {
    pick[key] = obj[key];
    return pick;
  }, {}) : {};
};
export var applyBridgeMiddleware = function applyBridgeMiddleware(_ref) {
  var moduleInstance = _ref.moduleInstance,
      globalInstance = _ref.globalInstance;

  if (!globalInstance) {
    return false;
  }

  if (moduleInstance === globalInstance) {
    return true;
  }

  if (moduleInstance.trackGlobalNamespaces && moduleInstance.trackGlobalNamespaces.length) {
    return true;
  }

  if (typeof moduleInstance.dispatchToGlobal === "function") {
    return true;
  }

  return false;
};
export var ensureAncestorRender = function ensureAncestorRender(store) {
  var listeners = {};

  var subscribe = function subscribe(listener) {
    var renderLevel = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "last";
    var id = Symbol("listener");
    listeners[renderLevel] = _objectSpread({}, listeners[renderLevel], _defineProperty({}, id, listener));
    return function () {
      // eslint-disable-next-line no-unused-vars
      var _ref2 = listeners[renderLevel] || {},
          remove = _ref2[id],
          rest = _objectWithoutProperties(_ref2, [id].map(_toPropertyKey));

      listeners[renderLevel] = rest;
    };
  };

  var execute = function execute() {
    var lastListeners = listeners.last;
    var len = Object.keys(listeners).length - (lastListeners ? 1 : 0);

    var _loop = function _loop(i) {
      var levelListeners = listeners[i.toString()];
      Object.getOwnPropertySymbols(levelListeners).forEach(function (sym) {
        return levelListeners[sym]();
      });
    };

    for (var i = 0; i < len; i++) {
      _loop(i);
    }

    Object.getOwnPropertySymbols(listeners.last || {}).forEach(function (sym) {
      return listeners.last[sym]();
    });
  };

  var subscription = noop;

  var unsubscribeExecute = function unsubscribeExecute() {
    subscription();
    subscription = noop;
  };

  var subscribeExecute = function subscribeExecute() {
    subscription = store.subscribe(execute);
  };

  return _objectSpread({}, store, {
    subscribe: subscribe,
    subscribeExecute: subscribeExecute,
    unsubscribeExecute: unsubscribeExecute
  });
};