"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var R = _interopRequireWildcard(require("ramda"));

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

var _providers = require("./providers");

var _consumers = require("./consumers");

var _index = _interopRequireDefault(require("Examples/ui-kit/Counter/index"));

var _model = require("Examples/ui-kit/Counter/model");

describe("## Redux-MVC connect", function () {
  it("Should pass the context instance as props", function () {
    var Div = (0, _consumers.connect)()(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div"
        }, props.instanceId)
      );
    });
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref) {
      var children = _ref.children;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, children)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, {
      instanceId: "contextInstanceId"
    },
    /*#__PURE__*/
    _react["default"].createElement(Div, null)));
    expect(wrapper.find("#div").text()).toBe("contextInstanceId");
  });
  it("Should pass the default instance as props", function () {
    var Div = (0, _consumers.connect)()(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div"
        }, props.instanceId)
      );
    });
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref2) {
      var children = _ref2.children;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, children)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, null,
    /*#__PURE__*/
    _react["default"].createElement(Div, null)));
    expect(wrapper.find("#div").text()).toBe("default");
  });
  it("Should pass the connected state", function () {
    var Div = (0, _consumers.connect)({
      count: _model.getters.count
    })(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref3) {
      var children = _ref3.children;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, children)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, null,
    /*#__PURE__*/
    _react["default"].createElement(Div, null)));
    var instance = wrapper.instance();
    instance.store.dispatch(_model.actions.add());
    expect(wrapper.find("#div").text()).toBe("1");
  });
  it("Should re-render when the state is updated", function () {
    var spy = jest.fn(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var Div = (0, _consumers.connect)({
      count: _model.getters.count
    })(spy);
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref4) {
      var children = _ref4.children;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, children)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, null,
    /*#__PURE__*/
    _react["default"].createElement(Div, null)));
    expect(wrapper.find("#div").text()).toBe("0");
    expect(spy.mock.calls.length).toBe(1);
    var instance = wrapper.instance();
    instance.store.dispatch(_model.actions.add());
    expect(wrapper.find("#div").text()).toBe("1");
    expect(spy.mock.calls.length).toBe(2);
  });
  it("Should re-render if props change", function () {
    var spy = jest.fn(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var decorate = R.compose((0, _providers.createContext)({
      module: _index["default"]
    }), (0, _consumers.connect)());
    var Div = decorate(spy);
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(Div, {
      count: 0
    }));
    expect(spy.mock.calls.length).toBe(1);
    expect(wrapper.find("#div").text()).toBe("0");
    wrapper.setProps({
      count: 1
    });
    expect(wrapper.find("#div").text()).toBe("1");
    expect(spy.mock.calls.length).toBe(2);
  });
  it("Should not re-render when the state is updated but the stateProps don't change", function () {
    var spy = jest.fn(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var Div = (0, _consumers.connect)({
      count: _model.getters.count
    })(spy);
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref5) {
      var children = _ref5.children;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, children)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, null,
    /*#__PURE__*/
    _react["default"].createElement(Div, {
      instanceId: "counter"
    })));
    expect(wrapper.find("#div").text()).toBe("0");
    expect(spy.mock.calls.length).toBe(1);
    var instance = wrapper.instance();
    instance.store.dispatch(_model.actions.add(null, {
      meta: {
        instanceId: "counter2"
      }
    }));
    expect(wrapper.find("#div").text()).toBe("0");
    expect(spy.mock.calls.length).toBe(1);
  });
  it("Should pass the connected state from the context instance", function () {
    var Div = (0, _consumers.connect)({
      count: _model.getters.count
    })(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref6) {
      var children = _ref6.children;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, children)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, {
      instanceId: "instance1"
    },
    /*#__PURE__*/
    _react["default"].createElement(Div, null)));
    var instance = wrapper.instance();
    instance.store.dispatch(_model.actions.add(null, {
      meta: {
        instanceId: "instance1"
      }
    }));
    expect(wrapper.find("#div").text()).toBe("1");
  });
  it("Should pass the connected state from the instance prop", function () {
    var Div = (0, _consumers.connect)({
      count: _model.getters.count
    })(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div"
        }, props.count)
      );
    });
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref7) {
      var children = _ref7.children;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, children)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, {
      instanceId: "instance1"
    },
    /*#__PURE__*/
    _react["default"].createElement(Div, {
      instanceId: "instance2"
    })));
    var instance = wrapper.instance();
    instance.store.dispatch(_model.actions.add(null, {
      meta: {
        instanceId: "instance2"
      }
    }));
    expect(wrapper.find("#div").text()).toBe("1");
  });
  it("Should pass the connected action", function () {
    var Div = (0, _consumers.connect)(null, {
      onClick: _model.actions.add
    })(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div",
          onClick: props.onClick
        })
      );
    });
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref8) {
      var children = _ref8.children;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, children)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, null,
    /*#__PURE__*/
    _react["default"].createElement(Div, null)));
    wrapper.find("#div").simulate("click");
    var instance = wrapper.instance();
    expect(instance.store.getState().Counter).toEqual({
      "default": {
        count: 1
      }
    });
  });
  it("Should dispatch to the context instance", function () {
    var Div = (0, _consumers.connect)(null, {
      onClick: _model.actions.add
    })(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div",
          onClick: props.onClick
        })
      );
    });
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref9) {
      var children = _ref9.children;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, children)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, {
      instanceId: "instance1"
    },
    /*#__PURE__*/
    _react["default"].createElement(Div, null)));
    wrapper.find("#div").simulate("click");
    var instance = wrapper.instance();
    expect(instance.store.getState().Counter).toEqual({
      "default": {
        count: 0
      },
      instance1: {
        count: 1
      }
    });
  });
  it("Should dispatch to the instance prop", function () {
    var Div = (0, _consumers.connect)(null, {
      onClick: _model.actions.add
    })(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div",
          onClick: props.onClick
        })
      );
    });
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref10) {
      var children = _ref10.children;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, children)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, {
      instanceId: "instance1"
    },
    /*#__PURE__*/
    _react["default"].createElement(Div, {
      instanceId: "instance2"
    })));
    wrapper.find("#div").simulate("click");
    var instance = wrapper.instance();
    expect(instance.store.getState().Counter).toEqual({
      "default": {
        count: 0
      },
      instance2: {
        count: 1
      }
    });
  });
  it("should call getters if props change", function () {
    var fixedCount = jest.fn(function () {
      return 10;
    });
    var Div = (0, _consumers.connect)({
      fixedCount: fixedCount
    })(function (props) {
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", {
          id: "div"
        }, props.text)
      );
    });
    var App = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref11) {
      var text = _ref11.text;
      return (
        /*#__PURE__*/
        _react["default"].createElement(Div, {
          text: text
        })
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(App, {
      text: "hola"
    }));
    expect(wrapper.find("#div").text()).toBe("hola");
    expect(fixedCount.mock.calls.length).toBe(1);
    wrapper.setProps({
      text: "chau"
    });
    expect(wrapper.find("#div").text()).toBe("chau");
    expect(fixedCount.mock.calls.length).toBe(2);
  });
});