"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _objectSpread3 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var R = _interopRequireWildcard(require("ramda"));

var _model = _interopRequireDefault(require("Examples/ui-kit/Counter/model"));

var _model2 = _interopRequireDefault(require("Examples/ui-kit/SearchBar/model"));

var _decorators = require("./decorators");

var _constants = require("./constants");

describe("## Redux-MVC decorators", function () {
  describe("### addReducer", function () {
    it("Should create the reducer", function () {
      var module = (0, _decorators.addReducer)()(_model["default"]);
      expect((0, _typeof2["default"])(module.reducer)).toBe("function");
    });
    it("Should return the ini state", function () {
      var module = (0, _decorators.addReducer)()(_model["default"]);
      var iniState = module.reducer(undefined, {});
      expect(iniState).toBe(module.iniState);
    });
    it("Should execute the right reducer", function () {
      var module = (0, _decorators.addReducer)()(_model["default"]);
      var nextState = module.reducer(module.iniState, module.actions.setCount(1));
      expect(nextState[module.namespace][_constants.DEFAULT_INSTANCE_ID]).toEqual({
        count: 1
      });
    });
    it("Should execute the right reducer and instance", function () {
      var module = (0, _decorators.addReducer)()(_model["default"]);
      var nextState = module.reducer(module.iniState, module.actions.setCount(1, {
        meta: {
          instanceId: "counter"
        }
      }));
      expect(nextState[module.namespace].counter).toEqual({
        count: 1
      });
    });
    it("Should return the previous state if the `action.type` handler is not found", function () {
      var module = (0, _decorators.addReducer)()(_model["default"]);
      var prevState = {};
      var nextState = module.reducer(prevState, {
        type: "fake"
      });
      expect(nextState).toBe(prevState);
    });
    it("Should ignore the instanceId if the model is a singleton", function () {
      var module = (0, _decorators.addReducer)()(_model2["default"]);
      var nextState = module.reducer(module.iniState, _model2["default"].actions.setSearch("hola", {
        meta: {
          instanceId: "searchBar"
        }
      }));
      expect(nextState[module.namespace][_constants.DEFAULT_INSTANCE_ID]).toEqual({
        search: "hola"
      });
    });
    it("Should merge the global updated state", function () {
      var module = (0, _decorators.addReducer)()(_model2["default"]);
      var nextState = module.reducer(module.iniState, {
        type: "action",
        payload: {
          newUser: "user"
        },
        meta: (0, _defineProperty2["default"])({}, _constants.GLOBAL_UPDATE, {
          user: "user"
        })
      });
      expect(nextState.user).toBe("user");
    });
    it("Should return the old state instance if the reducer is the identity", function () {
      var module = (0, _decorators.addReducer)()(_model2["default"]);
      var nextState = module.reducer(module.iniState, module.actions.identity());
      expect(nextState).toEqual(module.iniState);
    });
  });
  describe("### addBridge", function () {
    it("Should track all `module.dependencies` trackGlobalNamespaces and append the `trackGlobalNamespaces` param", function () {
      var mockModel = {
        dependencies: {
          searchBar: {
            trackGlobalNamespaces: ["App", "Routing"]
          },
          miniItemsSelector: {
            trackGlobalNamespaces: ["Event", "Routing"]
          }
        },
        namespace: "FakeModel"
      };
      var module = (0, _decorators.addBridge)({
        trackGlobalNamespaces: ["App", "User"]
      })(mockModel);
      expect(module.trackGlobalNamespaces.length).toEqual(4);
      expect(module.trackGlobalNamespaces).toEqual(expect.arrayContaining(["App", "User", "Routing", "Event"]));
    });
    it("Should track all `module.dependencies` trackGlobalNamespaces", function () {
      var mockModel = {
        dependencies: {
          searchBar: {
            trackGlobalNamespaces: ["App", "Routing"]
          },
          miniItemsSelector: {
            trackGlobalNamespaces: ["Event", "Routing"]
          }
        },
        namespace: "FakeModel"
      };
      var module = (0, _decorators.addBridge)()(mockModel);
      expect(module.trackGlobalNamespaces.length).toEqual(3);
      expect(module.trackGlobalNamespaces).toEqual(expect.arrayContaining(["App", "Routing", "Event"]));
    });
    it("Should set the `dispatchToGlobal` function", function () {
      var dispatchToGlobal = function dispatchToGlobal() {};

      var module = (0, _decorators.addBridge)({
        dispatchToGlobal: dispatchToGlobal
      })(_model2["default"]);
      expect(module).toEqual(expect.objectContaining({
        dispatchToGlobal: dispatchToGlobal
      }));
    });
    it("Should set the default `dispatchToGlobal` function when `dispatchToGlobal` param is empty", function () {
      var mockModel = {
        dependencies: {
          searchBar: {
            namespace: "SearchBar"
          },
          miniItemsSelector: {
            namespace: "MiniItemsSelector"
          }
        },
        namespace: "FakeModel"
      };
      var module = (0, _decorators.addBridge)()(mockModel);
      expect(module).toEqual(expect.objectContaining({
        dispatchToGlobal: expect.any(Function)
      }));
      var defaultDispatchToGlobal = module.dispatchToGlobal;
      expect(defaultDispatchToGlobal({
        type: "SearchBar/create"
      })).toEqual(false);
      expect(defaultDispatchToGlobal({
        type: "MiniItemsSelector/setName"
      })).toEqual(false);
      expect(defaultDispatchToGlobal({
        type: "FakeModel/init"
      })).toEqual(false);
      expect(defaultDispatchToGlobal({
        type: "App/commit"
      })).toEqual(true);
    });
    it("Should set the default `dispatchToGlobal` even if module dependencies is empty", function () {
      var mockModel = {
        namespace: "FakeModel"
      };
      var module = (0, _decorators.addBridge)()(mockModel);
      expect(module).toEqual(expect.objectContaining({
        dispatchToGlobal: expect.any(Function)
      }));
      var defaultDispatchToGlobal = module.dispatchToGlobal;
      expect(defaultDispatchToGlobal({
        type: "FakeModel/init"
      })).toEqual(false);
      expect(defaultDispatchToGlobal({
        type: "App/commit"
      })).toEqual(true);
    });
  });
  describe("### merge", function () {
    it("Should merge two modules", function () {
      var left = {
        namespace: "left",
        actions: "left",
        getters: "left",
        reducers: {
          left: ""
        },
        iniState: {
          left: ""
        },
        reducer: "left",
        dependencies: {
          Counter: _model["default"]
        },
        constructor: "left",
        componentDidCatch: "left",
        componentWillUnmount: "left",
        singleton: true,
        other: "left"
      };
      var right = {
        reducers: {
          right: ""
        },
        iniState: {
          right: ""
        },
        dependencies: {
          SearchBar: _model2["default"]
        }
      };
      var module = (0, _decorators.merge)(left)(right);
      expect(module).toEqual(expect.objectContaining((0, _objectSpread3["default"])({}, right, {
        dependencies: (0, _objectSpread3["default"])({}, left.dependencies, {}, right.dependencies, (0, _defineProperty2["default"])({}, left.namespace, left))
      })));
      expect(module).toEqual(expect.not.objectContaining(left));
    });
    it("Should merge two modules with no dependencies", function () {
      var module = (0, _decorators.merge)(_model2["default"])(_model["default"]);
      expect(module).toEqual(expect.objectContaining((0, _objectSpread3["default"])({}, _model["default"], {
        dependencies: (0, _defineProperty2["default"])({}, _model2["default"].namespace, _model2["default"])
      })));
    });
  });
  describe("### addLifecycle", function () {
    it("Should add `constructor` and `componentWillUnmount` to the module", function () {
      var newModule = R.compose((0, _decorators.addLifecycle)(), (0, _decorators.addReducer)())(_model["default"]);
      expect(newModule).toEqual(expect.objectContaining({
        constructor: expect.any(Function),
        componentWillUnmount: expect.any(Function)
      }));
    });
    it("Should create the store", function () {
      var newModule = R.compose((0, _decorators.addLifecycle)(), (0, _decorators.addReducer)())(_model["default"]);
      var contextId = "fakeContextId";
      var moduleInstances = (0, _defineProperty2["default"])({}, contextId, (0, _objectSpread3["default"])({}, newModule));
      var moduleInstance = newModule.constructor({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      expect(moduleInstance.store).toEqual(expect.objectContaining({
        getState: expect.any(Function),
        subscribe: expect.any(Function),
        dispatch: expect.any(Function)
      }));
      moduleInstance.store.dispatch(newModule.actions.add());
      expect(newModule.getters.count(moduleInstance.store.getState())).toEqual(1);
      moduleInstance.store.dispatch(newModule.actions.add(null, {
        meta: {
          instanceId: "fakeInstance"
        }
      }));
      expect(newModule.getters.count(moduleInstance.store.getState(), {
        instanceId: "fakeInstance"
      })).toEqual(1);
    });
    it("Should persist the store by default", function () {
      var newModule = R.compose((0, _decorators.addLifecycle)(), (0, _decorators.addReducer)())(_model["default"]);
      var contextId = "fakeContextId";
      var moduleInstances = (0, _defineProperty2["default"])({}, contextId, (0, _objectSpread3["default"])({}, newModule));
      var moduleInstance = newModule.constructor({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      var newModuleInstance = newModule.constructor({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      expect(moduleInstance.store.getState()).toEqual(newModuleInstance.store.getState());
    });
    it("Should dispatch to the global instance and track the result", function () {
      var _moduleInstances3;

      var searchBarModule = R.compose((0, _decorators.addLifecycle)(), (0, _decorators.addReducer)())(_model2["default"]);
      var counterModule = R.compose((0, _decorators.addLifecycle)(), (0, _decorators.addReducer)(), (0, _decorators.addBridge)({
        trackGlobalNamespaces: [searchBarModule.namespace]
      }))(_model["default"]);
      var contextId = "fakeContextId";
      var moduleInstances = (_moduleInstances3 = {}, (0, _defineProperty2["default"])(_moduleInstances3, contextId, (0, _objectSpread3["default"])({}, counterModule)), (0, _defineProperty2["default"])(_moduleInstances3, _constants.GLOBAL_CONTEXT_ID, (0, _objectSpread3["default"])({}, searchBarModule)), _moduleInstances3);
      var globalInstance = searchBarModule.constructor({
        moduleInstances: moduleInstances,
        contextId: _constants.GLOBAL_CONTEXT_ID
      });
      moduleInstances[_constants.GLOBAL_CONTEXT_ID] = globalInstance;
      var moduleInstance = counterModule.constructor({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      moduleInstance.store.dispatch(searchBarModule.actions.setSearch("hola"));
      expect(searchBarModule.getters.search(globalInstance.store.getState())).toEqual("hola");
      expect(searchBarModule.getters.search(moduleInstance.store.getState())).toEqual("hola");
    });
    it("Should unbind the bridge on `componentWillUnmount`", function () {
      var _moduleInstances4;

      var searchBarModule = R.compose((0, _decorators.addLifecycle)(), (0, _decorators.addReducer)())(_model2["default"]);
      var counterModule = R.compose((0, _decorators.addLifecycle)(), (0, _decorators.addReducer)(), (0, _decorators.addBridge)({
        trackGlobalNamespaces: [searchBarModule.namespace]
      }))(_model["default"]);
      var contextId = "fakeContextId";
      var moduleInstances = (_moduleInstances4 = {}, (0, _defineProperty2["default"])(_moduleInstances4, contextId, (0, _objectSpread3["default"])({}, counterModule)), (0, _defineProperty2["default"])(_moduleInstances4, _constants.GLOBAL_CONTEXT_ID, (0, _objectSpread3["default"])({}, searchBarModule)), _moduleInstances4);
      var globalInstance = searchBarModule.constructor({
        moduleInstances: moduleInstances,
        contextId: _constants.GLOBAL_CONTEXT_ID
      });
      moduleInstances[_constants.GLOBAL_CONTEXT_ID] = globalInstance;
      var moduleInstance = counterModule.constructor({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      moduleInstances[contextId] = moduleInstance;
      moduleInstance.store.dispatch(searchBarModule.actions.setSearch("hola"));
      counterModule.componentWillUnmount({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      moduleInstance.store.dispatch(searchBarModule.actions.setSearch("chau"));
      expect(searchBarModule.getters.search(globalInstance.store.getState())).toEqual("hola");
      expect(searchBarModule.getters.search(moduleInstance.store.getState())).toEqual("hola");
    });
  });
});