"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

var _providers = require("./providers");

var _context = require("./context");

var _constants = require("./constants");

var _index = _interopRequireDefault(require("Examples/ui-kit/Counter/index"));

describe("## Redux-MVC context", function () {
  it("Should pass the default instance id", function () {
    var Div = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref) {
      var instanceId = _ref.instanceId;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, instanceId)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(Div, null));
    expect(wrapper.text()).toContain(_constants.DEFAULT_INSTANCE_ID);
  });
  it("Should pass the instance id prop", function () {
    var Div = (0, _providers.createContext)({
      module: _index["default"]
    })(function (_ref2) {
      var instanceId = _ref2.instanceId;
      return (
        /*#__PURE__*/
        _react["default"].createElement("div", null, instanceId)
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(Div, {
      instanceId: "instanceId"
    }));
    expect(wrapper.text()).toContain("instanceId");
  });
  it("Should set the current contextId", function () {
    var Div = (0, _providers.createContext)({
      module: _index["default"],
      contextId: "context"
    })(function () {
      return (
        /*#__PURE__*/
        _react["default"].createElement(_context.StoreManager.Consumer, null, function (value) {
          return value.contextId;
        })
      );
    });
    var wrapper = (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(Div, null));
    expect(wrapper.text()).toContain("context");
  });
  it("Should create the module instance", function () {
    var moduleInstances;
    var Div = (0, _providers.createContext)({
      module: _index["default"],
      contextId: "context"
    })(function () {
      return (
        /*#__PURE__*/
        _react["default"].createElement(_context.StoreManager.Consumer, null, function (value) {
          moduleInstances = value.moduleInstances;
        })
      );
    });
    (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(Div, null));
    expect(moduleInstances).toEqual(expect.objectContaining({
      context: expect.any(Object)
    }));
  });
  it("Should create the store", function () {
    var moduleInstances;
    var Div = (0, _providers.createContext)({
      module: _index["default"],
      contextId: "context"
    })(function () {
      return (
        /*#__PURE__*/
        _react["default"].createElement(_context.StoreManager.Consumer, null, function (value) {
          moduleInstances = value.moduleInstances;
        })
      );
    });
    (0, _enzyme.mount)(
    /*#__PURE__*/
    _react["default"].createElement(Div, null));
    expect(moduleInstances.context.store).toEqual(expect.objectContaining({
      getState: expect.any(Function),
      subscribe: expect.any(Function)
    }));
  });
});