"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var R = _interopRequireWildcard(require("ramda"));

var _Counter = _interopRequireDefault(require("Examples/ui-kit/Counter"));

var _SearchBar = _interopRequireDefault(require("Examples/ui-kit/SearchBar"));

var _selectors = require("./selectors");

var _decorators = require("./decorators");

describe("## Create Selector", function () {
  it("Should return a selector function", function () {
    var selector = (0, _selectors.createSelector)(_Counter["default"].getters.count, R.inc);
    expect(selector).toEqual(expect.any(Function));
    expect((0, _typeof2["default"])(selector.id)).toEqual("symbol");
    expect(selector.dependencies).toEqual([_Counter["default"].getters.count]);
    expect(selector.resultFn).toEqual(R.inc);
    expect(selector.ownCache).toEqual(expect.objectContaining((0, _defineProperty2["default"])({}, selector.id, {})));
  });
  it("Should return the same function if no dependencies where passed", function () {
    var selector = (0, _selectors.createSelector)(_Counter["default"].getters.count);
    expect(selector).toEqual(_Counter["default"].getters.count);
  });
  it("Should compute the value once if dependencies dont change", function () {
    var count = jest.fn(_Counter["default"].getters.count);
    var resultFn = jest.fn(R.inc);
    var selector = (0, _selectors.createSelector)(count, resultFn);
    var value = selector(_Counter["default"].iniState);
    expect(value).toEqual(1);
    expect(count).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(1);
    var newValue = selector(_Counter["default"].iniState);
    expect(newValue).toEqual(1);
    expect(count).toHaveBeenCalledTimes(2);
    expect(resultFn).toHaveBeenCalledTimes(1);
  });
  it("Should re-compute the value if dependencies change", function () {
    var count = jest.fn(_Counter["default"].getters.count);
    var resultFn = jest.fn(R.inc);
    var selector = (0, _selectors.createSelector)(count, resultFn);
    var value = selector(_Counter["default"].iniState);
    expect(value).toEqual(1);
    expect(count).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(1);
    var newValue = selector(_Counter["default"].reducer(_Counter["default"].iniState, _Counter["default"].actions.add()));
    expect(newValue).toEqual(2);
    expect(count).toHaveBeenCalledTimes(2);
    expect(resultFn).toHaveBeenCalledTimes(2);
  });
  it("Should re-compute the value for a different instanceId", function () {
    var count = jest.fn(_Counter["default"].getters.count);
    var resultFn = jest.fn(R.inc);
    var selector = (0, _selectors.createSelector)(count, resultFn);
    var value = selector(_Counter["default"].iniState);
    expect(value).toEqual(1);
    expect(count).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(1);
    var instanceValue = selector(_Counter["default"].reducer(_Counter["default"].iniState, _Counter["default"].actions.add()), {
      instanceId: "counter"
    });
    expect(instanceValue).toEqual(1);
    expect(count).toHaveBeenCalledTimes(2);
    expect(resultFn).toHaveBeenCalledTimes(2);
  });
  it("Should compose selectors and call the dependencies and result functions when needed", function () {
    var module = (0, _decorators.addReducer)()((0, _decorators.merge)(_SearchBar["default"])(_Counter["default"]));
    var count = jest.fn(_Counter["default"].getters.count);
    var inc = jest.fn(R.compose(R.toString, R.inc));
    var oneMore = (0, _selectors.createSelector)(count, inc);
    var search = jest.fn(_SearchBar["default"].getters.search);
    var prepend = jest.fn(R.concat("el "));
    var greetings = (0, _selectors.createSelector)(search, prepend);
    var resultFn = jest.fn(R.concat);
    var onePlusGreetings = (0, _selectors.createSelector)(greetings, oneMore, resultFn);
    var value = onePlusGreetings(module.iniState);
    expect(value).toEqual("el 1");
    expect(count).toHaveBeenCalledTimes(1);
    expect(inc).toHaveBeenCalledTimes(1);
    expect(search).toHaveBeenCalledTimes(1);
    expect(prepend).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(1);
    var newValue = onePlusGreetings(module.reducer(module.iniState, _Counter["default"].actions.add()));
    expect(newValue).toEqual("el 2");
    expect(count).toHaveBeenCalledTimes(2);
    expect(inc).toHaveBeenCalledTimes(2);
    expect(search).toHaveBeenCalledTimes(2);
    expect(prepend).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(2);
  });
  it("Should use the cache parameter", function () {
    var _expect$objectContain2;

    var module = (0, _decorators.addReducer)()((0, _decorators.merge)(_SearchBar["default"])(_Counter["default"]));
    var count = jest.fn(_Counter["default"].getters.count);
    var inc = jest.fn(R.compose(R.toString, R.inc));
    var oneMore = (0, _selectors.createSelector)(count, inc);
    var search = jest.fn(_SearchBar["default"].getters.search);
    var prepend = jest.fn(R.concat("el "));
    var greetings = (0, _selectors.createSelector)(search, prepend);
    var resultFn = jest.fn(R.concat);
    var onePlusGreetings = (0, _selectors.createSelector)(greetings, oneMore, resultFn);
    var cache = {};
    onePlusGreetings(module.iniState, {}, cache);
    expect(cache).toEqual(expect.objectContaining((_expect$objectContain2 = {}, (0, _defineProperty2["default"])(_expect$objectContain2, onePlusGreetings.id, expect.any(Object)), (0, _defineProperty2["default"])(_expect$objectContain2, greetings.id, expect.any(Object)), (0, _defineProperty2["default"])(_expect$objectContain2, oneMore.id, expect.any(Object)), _expect$objectContain2)));
  });
  it("Should recompute the resultFn with a new cache", function () {
    var module = (0, _decorators.addReducer)()((0, _decorators.merge)(_SearchBar["default"])(_Counter["default"]));
    var count = jest.fn(_Counter["default"].getters.count);
    var inc = jest.fn(R.compose(R.toString, R.inc));
    var oneMore = (0, _selectors.createSelector)(count, inc);
    var search = jest.fn(_SearchBar["default"].getters.search);
    var prepend = jest.fn(R.concat("el "));
    var greetings = (0, _selectors.createSelector)(search, prepend);
    var resultFn = jest.fn(R.concat);
    var onePlusGreetings = (0, _selectors.createSelector)(greetings, oneMore, resultFn);
    var cache = {};
    onePlusGreetings(module.iniState, {}, cache);
    expect(count).toHaveBeenCalledTimes(1);
    expect(inc).toHaveBeenCalledTimes(1);
    expect(search).toHaveBeenCalledTimes(1);
    expect(prepend).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(1);
    var newCache = {};
    onePlusGreetings(module.iniState, {}, newCache);
    expect(count).toHaveBeenCalledTimes(2);
    expect(inc).toHaveBeenCalledTimes(2);
    expect(search).toHaveBeenCalledTimes(2);
    expect(prepend).toHaveBeenCalledTimes(2);
    expect(resultFn).toHaveBeenCalledTimes(2);
  });
});