"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ensureAncestorRender = exports.applyBridgeMiddleware = exports.pick = exports.uniq = exports.diff = exports.isEmpty = exports.capitalizeFirst = exports.getDisplayName = exports.getActionInstanceId = exports.getSelectorInstanceId = exports.mergeAll = exports.has = exports.path = exports.pathOr = exports.prop = exports.propOr = exports.noop = exports.identity = void 0;

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _toPropertyKey2 = _interopRequireDefault(require("@babel/runtime/helpers/toPropertyKey"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _objectSpread4 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _constants = require("./constants");

/* eslint-disable no-underscore-dangle */
var identity = function identity(x) {
  return x;
};

exports.identity = identity;

var noop = function noop() {};

exports.noop = noop;

var propOr = function propOr(or, prop, val) {
  if (val && typeof val[prop] !== "undefined" && val[prop] !== null) {
    return val[prop];
  }

  return or;
};

exports.propOr = propOr;

var prop = function prop(_prop, val) {
  if (val) {
    return val[_prop];
  }

  return undefined;
};

exports.prop = prop;

var pathOr = function pathOr(or, path, val) {
  var result = path.reduce(function (acc, key) {
    return prop(key, acc);
  }, val);
  return typeof result !== "undefined" && result !== null ? result : or;
};

exports.pathOr = pathOr;

var path = function path(_path, val) {
  return _path.reduce(function (acc, key) {
    return prop(key, acc);
  }, val);
};

exports.path = path;
var EMPTY = Symbol("empty");

var has = function has() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return pathOr.apply(void 0, [EMPTY].concat(args)) !== EMPTY;
};

exports.has = has;

var mergeAll = function mergeAll(all) {
  return all.reduce(function (acc, val) {
    return (0, _objectSpread4["default"])({}, acc, {}, val || {});
  }, {});
}; // mvc utils


exports.mergeAll = mergeAll;

var getSelectorInstanceId = function getSelectorInstanceId(props, singleton) {
  return singleton ? _constants.DEFAULT_INSTANCE_ID : propOr(_constants.DEFAULT_INSTANCE_ID, "instanceId", props);
};

exports.getSelectorInstanceId = getSelectorInstanceId;

var getActionInstanceId = function getActionInstanceId(action, singleton) {
  return singleton ? _constants.DEFAULT_INSTANCE_ID : pathOr(_constants.DEFAULT_INSTANCE_ID, ["meta", "instanceId"], action);
};

exports.getActionInstanceId = getActionInstanceId;

var getDisplayName = function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || "Component";
};

exports.getDisplayName = getDisplayName;

var capitalizeFirst = function capitalizeFirst(str) {
  return str[0].toUpperCase() + str.slice(1);
};

exports.capitalizeFirst = capitalizeFirst;

var isEmpty = function isEmpty(obj) {
  return !obj || Object.keys(obj).length === 0;
};

exports.isEmpty = isEmpty;

var diff = function diff(oldObj, newObj, keys) {
  if (oldObj === newObj) {
    return false;
  }

  if ((0, _typeof2["default"])(oldObj) === "object" && (0, _typeof2["default"])(newObj) === "object") {
    if (Array.isArray(keys)) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = keys[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var key = _step.value;

          if (oldObj[key] !== newObj[key]) {
            return true;
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator["return"] != null) {
            _iterator["return"]();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return false;
    }

    if (Object.keys(oldObj).length !== Object.keys(newObj).length) {
      return true;
    }

    for (var _key2 in oldObj) {
      if (oldObj[_key2] !== newObj[_key2]) {
        return true;
      }
    }
  }

  return false;
};

exports.diff = diff;

var uniq = function uniq(arr) {
  return Object.values(arr.reduce(function (acc, key) {
    return (0, _objectSpread4["default"])({}, acc, (0, _defineProperty2["default"])({}, key, key));
  }, {}));
};

exports.uniq = uniq;

var pick = function pick(arr, obj) {
  return Array.isArray(arr) && obj ? arr.reduce(function (pick, key) {
    pick[key] = obj[key];
    return pick;
  }, {}) : {};
};

exports.pick = pick;

var applyBridgeMiddleware = function applyBridgeMiddleware(_ref) {
  var moduleInstance = _ref.moduleInstance,
      globalInstance = _ref.globalInstance;

  if (!globalInstance) {
    return false;
  }

  if (moduleInstance === globalInstance) {
    return true;
  }

  if (moduleInstance.trackGlobalNamespaces && moduleInstance.trackGlobalNamespaces.length) {
    return true;
  }

  if (typeof moduleInstance.dispatchToGlobal === "function") {
    return true;
  }

  return false;
};

exports.applyBridgeMiddleware = applyBridgeMiddleware;

var ensureAncestorRender = function ensureAncestorRender(store) {
  var listeners = {};

  var subscribe = function subscribe(listener) {
    var renderLevel = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "last";
    var id = Symbol("listener");
    listeners[renderLevel] = (0, _objectSpread4["default"])({}, listeners[renderLevel], (0, _defineProperty2["default"])({}, id, listener));
    return function () {
      // eslint-disable-next-line no-unused-vars
      var _ref2 = listeners[renderLevel] || {},
          remove = _ref2[id],
          rest = (0, _objectWithoutProperties2["default"])(_ref2, [id].map(_toPropertyKey2["default"]));

      listeners[renderLevel] = rest;
    };
  };

  var execute = function execute() {
    var lastListeners = listeners.last;
    var len = Object.keys(listeners).length - (lastListeners ? 1 : 0);

    var _loop = function _loop(i) {
      var levelListeners = listeners[i.toString()];
      Object.getOwnPropertySymbols(levelListeners).forEach(function (sym) {
        return levelListeners[sym]();
      });
    };

    for (var i = 0; i < len; i++) {
      _loop(i);
    }

    Object.getOwnPropertySymbols(listeners.last || {}).forEach(function (sym) {
      return listeners.last[sym]();
    });
  };

  var subscription = noop;

  var unsubscribeExecute = function unsubscribeExecute() {
    subscription();
    subscription = noop;
  };

  var subscribeExecute = function subscribeExecute() {
    subscription = store.subscribe(execute);
  };

  return (0, _objectSpread4["default"])({}, store, {
    subscribe: subscribe,
    subscribeExecute: subscribeExecute,
    unsubscribeExecute: unsubscribeExecute
  });
};

exports.ensureAncestorRender = ensureAncestorRender;